import os
import sys
from collections import namedtuple

ERROR_INFO = namedtuple('ERROR_INFO', ['File', 'Line'])


def get_error_info():
    exc_type, exc_obj, exc_tb = sys.exc_info()
    file_name = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
    return ERROR_INFO(File=file_name, Line=exc_tb.tb_lineno)



import json
import unittest
from os.path import normpath

from yaml_walker.__main__ import run_cli, parse
from yaml_walker.api import Ypath


class Test_Ypath(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls._path = normpath(r'./example.yaml')

    def test_simple(self):
        pattern = 'node.nd_1.data'
        result = run_cli([pattern, self._path])
        print(json.dumps(result, sort_keys=True, indent=4))

    def test_comparer(self):
        pattern = 'node.nd_1.data[id=2]'
        result = run_cli([pattern, self._path])
        print(json.dumps(result, sort_keys=True, indent=4))

    def test_comparer_with_gt(self):
        pattern = 'node.nd_2.data[id>=2]'
        result = run_cli([pattern, self._path])
        print(json.dumps(result, sort_keys=True, indent=4))

    def test_comparer_with_sub_node(self):
        pattern = 'node.nd_1.data[id=2]sub_data'
        result = run_cli([pattern, self._path])
        print(json.dumps(result, sort_keys=True, indent=4))

    def test_comparer_with_regex(self):
        pattern = 'node.nd_+'
        result = run_cli([pattern, self._path])
        print(json.dumps(result, sort_keys=True, indent=4))

    def test_custom_list(self):
        data = [{
                    'alias': 'login',
                    'prompt': 'login:',
                    'command': 'root'
                },
                {
                    'alias': 'password',
                    'prompt': 'Password:',
                    'command': 'S-N<&t8{<wu98wCD'
                },
                {
                    'alias': 'shell',
                    'prompt': '#',
                    'command': None
                },
                {
                    'alias': 'exit',
                    'command': 'exit',
                    'prompt': 'login:'
                }
                ]
        pattern = '[alias=login]'
        result = Ypath(pattern)(data)
        print(json.dumps(result, sort_keys=True, indent=4))


class Test_YDict(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls._path = r'.\example.yaml'
        cls.y_dict = parse(cls._path)

    def test_simple(self):
        result = self.y_dict.node.nd_1.data
        self.assertNotEqual(result, "Empty!!!")
        print(json.dumps(result.__dict__(), sort_keys=True, indent=4))

    def test_comparer(self):
        y_dict = parse(self._path)
        result = y_dict.node.nd_1.data['id=2']
        self.assertNotEqual(result, "Empty!!!")
        print(json.dumps(result.__dict__(), sort_keys=True, indent=4))

    def test_comparer_with_gt(self):
        y_dict = parse(self._path)
        result = y_dict.node.nd_1.data['id>2']
        self.assertNotEqual(result, "Empty!!!")
        print(json.dumps(result.__dict__(), sort_keys=True, indent=4))

    def test_comparer_with_sub_node(self):
        y_dict = parse(self._path)
        result = y_dict.node.nd_1.data['id=2'].sub_data
        self.assertNotEqual(result, "Empty!!!")
        print(json.dumps(result.__dict__(), sort_keys=True, indent=4))


if __name__ == '__main__':
    unittest.main()




import yaml_walker.api as api
from .__main__ import run_cli, parse, query


__version__ = '1.0.21'
__author__ = 'Dmitry Oguz'
__author_email__ = 'doguz2509@gmail.com'
__git_url__ = r'https://github.com/doguz2509/YamlWalker'


__all__ = [
    run_cli.__name__,
    parse.__name__,
    query.__name__,
    'api'
]




import re

from yaml_walker.tools.comparer import Comparer
from yaml_walker.tools.get_error_info import get_error_info
# import logging
# logger = logging.getLogger(__name__)


class node_lookup:
    def __init__(self, element):
        self._element = element
        # logger.debug(f"Element: {self._element}")
        assert self._element is not None and self._element != '', "Empty element provided"
        self._wild_card_pattern = re.findall(r'[+|*|?]+', self._element)

    def __str__(self):
        return f"{self._element} "

    @property
    def shall_be_last_pattern(self):
        return bool(self._wild_card_pattern)

    def _dict_lookup(self, data):
        try:
            if not self._wild_card_pattern:
                if self._element == ' ':
                    return list(data.values())
                if isinstance(data, dict):
                    assert self._element in data.keys()
                    return data[self._element]
                elif isinstance(data, list):
                    _res = []
                    for _elem in data:
                        _res.append(self(_elem))
                    return _res
            else:
                replacements = {r'+': r'.+', r'*': r'.*', r'?': r"[\s\d_\-]{1}"}
                expression = self._element
                _result = {}
                for _str, _pattern in replacements.items():
                    expression = expression.replace(_str, _pattern)
                for k, v in data.items():
                    if re.match(expression, k):
                        _result.update({k: v})
                return _result
        except AssertionError:
            raise KeyError(self._element)

    def _list_lookup(self, _data: list):
        _res = []
        assert isinstance(_data, (list, tuple)), f"Data expected to by list (Real: {_data})"
        try:
            ec = Comparer(self._element)
            for _elem in _data:
                for k_elem, v_elem in _elem.items():
                    if k_elem == ec.parameter:
                        if ec(v_elem):
                            _res.append(_elem)
            return _res[0] if len(_res) == 1 else _res
        except Exception as e:
            f, l = get_error_info()
            raise type(e)(f"{e}; File: {f}:{l}")

    def __call__(self, data):
        if isinstance(data, dict):
            return self._dict_lookup(data)
        elif isinstance(data, (list, tuple)):
            return self._list_lookup(data)




import re

from yaml_walker.api.lookup import node_lookup
from yaml_walker.tools.get_error_info import get_error_info
import logging
logger = logging.getLogger(__name__)


class YQueryError(KeyError):
    pass


class Ypath:
    EXPRESION_SEPARATOR = r'([\.|\[\]])+'
    DELIMITERS = ['.', '[', ']']

    def __init__(self, path_pattern):
        self._node_path = []
        self._base_expression = ''
        self.parse(path_pattern)
        print("")

    def __str__(self):
        return f"{self.__class__.__name__}: {self._base_expression}"

    def parse(self, path_string: str):
        self._base_expression = path_string
        delimited_pattern = [st for st in re.split(self.EXPRESION_SEPARATOR, path_string) if st != '']
        # logger.debug(f"Pattern: {', '.join(delimited_pattern)}")
        for index, item in enumerate(delimited_pattern):
            if item in self.DELIMITERS:
                continue

            call_back = node_lookup(item)
            if call_back.shall_be_last_pattern and \
                    index < len(list([i for i in delimited_pattern if i not in self.DELIMITERS])):
                raise YQueryError(f"Wrong pattern; Wildcard element must be last one ({path_string})")
            self._node_path.append(call_back)

    def __call__(self, data: dict, return_name=False):
        try:
            _temp_data = data
            for cb in self._node_path:
                _temp_data = cb(_temp_data)
            return _temp_data
        except KeyError as e:
            raise YQueryError(f"Pattern '{self._base_expression}' error occur element '{e.args[0]}'")
        except Exception as e:
            f, l = get_error_info()
            raise Exception(f"Unexpected error: {e}; File: {f}:{l}")



from yaml_walker.api import Ypath
import logging
logger = logging.getLogger(__name__)


class YDict:
    def __init__(self, data: dict, item=None):
        self._node: dict = data if item is None else Ypath(item)(data)

    def _get_y_query_item(self, item):
        if isinstance(item, YDict):
            temp_query = self._node
        else:
            temp_query = Ypath(item)
        return YDict(temp_query(self._node))

    def __str__(self):
        return str(self._node)

    def __getattr__(self, item):
        return self._get_y_query_item(item)

    def __getitem__(self, item):
        return self._get_y_query_item(item)

    def __dict__(self):
        return self._node

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_type is None:
            return None
        return exc_type(exc_val, exc_tb)


import operator
import re

PATTERN_REGEX = re.compile(r'([\-\w]+)([\!<>=]{1,2})(.+)')
_OPERATIONS = {
            '=': operator.eq,
            '==': operator.eq,
            '!=': operator.ne,
            '<': operator.lt,
            '<=': operator.le,
            '>': operator.gt,
            '>=': operator.ge
        }


class Comparer:
    def __init__(self, expression: str):
        m = PATTERN_REGEX.match(expression)
        self._expression = expression
        self._parameter = m.group(1)
        self._operator = _OPERATIONS[m.group(2)]
        try:
            _expected = int(m.group(3))
        except ValueError:
            try:
                _expected = float(m.group(3))
            except ValueError:
                _expected = m.group(3)
        self._expected = _expected

    @property
    def parameter(self):
        return self._parameter

    def __str__(self):
        return self._expression

    def __call__(self, value):
        return self._operator(value, self._expected)



from os.path import normpath
from setuptools import setup, find_packages
from yaml_walker import __version__, __author__, __author_email__, __git_url__

with open(normpath('README.md'), 'r') as rm:
    long_description = rm.read()

setup(
    name='yaml_walker',
    version=__version__,
    packages=find_packages(),
    url=__git_url__,
    license='MIT',
    author=__author__,
    author_email=__author_email__,
    description='Data walker (in dot notation style) & filter (Xpath analog) for yaml look data structure',
    long_description_content_type="text/markdown",
    long_description=long_description,
    install_requires=['PyYAML'],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8'
    ],
    package_data={
        '': [
            'unittests/*.yaml'
        ]
    }
)



import json
import sys

import yaml
import yaml_walker
from yaml_walker.api import Ypath, YDict


def query(pattern, data):
    return Ypath(pattern)(data)


def parse(yaml_path):
    with open(yaml_path) as fr:
        data = yaml.load(fr, yaml.FullLoader)
        return YDict(data)


def run_cli(argv):
    pattern = argv[0]
    yaml_path = argv[1]
    with open(yaml_path) as fr:
        data = yaml.load(fr, yaml.FullLoader)
        return query(pattern, data)


if __name__ == '__main__':
    if sys.argv[1] in ['-v', '--version']:
        print(f"yaml_walker: {yaml_walker.__version__}")
        sys.exit(-1)
    result = run_cli(sys.argv[1:])
    print(json.dumps(result, sort_keys=True, indent=4))
